/*
 * SPIHandler.c
 *
 *  Created on: 20.12.2012
 *      Author: kurt
 */

#include "stdio.h"
#include "string.h"
#include "avr/io.h"
#include "inttypes.h"
#include "avr/interrupt.h"
#include "SPIHandlerDefinitions.h"
#include "ButtonHandler.h"
#include "TouchDriver.h"
#include "LCDDriver.h"
#include "LEDDriver.h"

#define STX 0x02
#define ETX 0x03
#define ENQ 0x05
#define ACK 0x06
#define NACK 0x15
#define ESC 0x1b

#define PROTO_WAIT_STX 0
#define PROTO_READ_DATA 1
#define PROTO_READ_ESCAPED 2
#define PROTO_ETX_RECEIVED 3

static uint8_t protocolState;
static uint8_t bufferIndex;
static uint8_t buffer[10];

void SPI_init(void) {
	protocolState = PROTO_WAIT_STX;
	bufferIndex = 0;
	DDRB |= (0 << KKDISPLAY_CLK) | (1 << KKDISPLAY_MISO) | (0 << KKDISPLAY_MOSI) | (0 << KKDISPLAY_SELECT);
	SPCR = (1 << SPIE) | (1 << SPE) | (0 << DORD) | (0 << MSTR) | (1 << CPOL) | (1 << CPHA) | (1 << SPR0) | (0 << SPR1);
	DDRD |= (1 << KKDISPLAY_REQ);
	PORTD &= ~(1 << KKDISPLAY_REQ);
}

void SPI_sendButton(uint8_t lb) {
	SPDR = lb;
	PORTD |= (1 << KKDISPLAY_REQ);
	PORTD &= ~(1 << KKDISPLAY_REQ);
}

ISR(SPI_STC_vect) {
	uint8_t byte;
	byte = SPDR;
	if (byte == ENQ) return;
	switch (protocolState) {
	case PROTO_WAIT_STX:
		if (byte == STX) {
			bufferIndex = 0;
			protocolState = PROTO_READ_DATA;
		}
		break;
	case PROTO_READ_DATA:
		if (byte == ESC) {
			protocolState = PROTO_READ_ESCAPED;
		} else if (byte == ETX) {
			protocolState = PROTO_ETX_RECEIVED;
		} else {
			buffer[bufferIndex++] = byte;
		}
		break;
	case PROTO_READ_ESCAPED:
		buffer[bufferIndex++] = byte - 0x20;
		protocolState = PROTO_READ_DATA;
		break;
	case PROTO_ETX_RECEIVED:
		break;
	}
}

void SPI_handleBuffer(void) {
	uint8_t lrc;
	uint8_t lrci;
	if (protocolState != PROTO_ETX_RECEIVED) return;
	lrc = 0;
	for (lrci = 0; lrci < bufferIndex - 1; lrci++) {
		lrc += buffer[lrci];
	}
	if (lrc != buffer[bufferIndex - 1]) {
		bufferIndex = 0;
		return;
	}
	switch (buffer[0]) {
	case 0x00:					// set Color
		LED_setColor(buffer[1], buffer[2], buffer[3]);
		break;
	case 0x01:					// restart LCD
		Button_init();
		LED_init();
		LCD_init();
		LED_setColor(255, 255, 255);
		LCD_loop(1);
		break;
	case 0x02:					// initialize LCD
		Touch_clearAll();
		Button_init();
		LED_init();
		LCD_init();
		Touch_setup();
		LCD_loop(1);
		break;
	case 0x03:					// set LCD to logo + button printf
		LCD_setPrintLogoMode();
		break;
	case 0x04:					// set button for printf
		Button_setPrintButton(buffer[1], buffer[2], buffer[3]);
		break;
	case 0x05:					// set LCD to simple printf
		LCD_setPrintfMode();
		break;
	case 0x06:					// set LCD to button printf
		LCD_setPrintButtonMode();
		break;
	case 0x07:					// enable graphics
		LCD_enableGraphics(buffer[1]);
		break;
	case 0x08:					// set values
		LCD_setGraphicsData(buffer[1], buffer[2], buffer[3]);
		break;
	case 0x09:
		LCD_putBuffer(buffer[1]);
		break;
	}
	bufferIndex = 0;
	protocolState = PROTO_WAIT_STX;
	SPI_sendButton(0xfe);
}
