/*
 * LCDDriver.c
 *
 *  Created on: 16.02.2013
 *      Author: kurt
 */

#include "stdlib.h"
#include "stdio.h"
#include "TouchDriverDefinitions.h"
#include "TouchDriver.h"
#include "LEDDriver.h"
#include "LCDDriver.h"
#include "EEPROMHandler.h"
#include "string.h"
#include "avr/io.h"

#define WIDTH 160L
#define HEIGHT 104L

#define TOUCH_SAMPLE TOUCH_TOP
#define TOUCH_GND TOUCH_LEFT

#define AVERAGE 4

static uint16_t buttonX1, buttonX2, buttonX3;
static uint16_t buttonY1, buttonY2;

static uint8_t debounceTouch, debounceButton1, debounceButton2;
static int16_t lastX = 0, lastY = 0;

static uint8_t touched;
static int16_t lowerX, upperX, lowerY, upperY;

static void setupButtons(void) {
	uint16_t xunit = (upperX - lowerX) / 4;
	uint16_t yunit = (upperY - lowerY) / 3;
	buttonY1 = lowerY + yunit;
	buttonY2 = lowerY + 2 * yunit;
	buttonX1 = lowerX + xunit;
	buttonX2 = lowerX + 2 * xunit;
	buttonX3 = lowerX + 3 * xunit;
}

uint8_t Touch_init(void) {
	int16_t tmp[4];
	DDRA &= ~((1 << TOUCH_LEFT) | (1 << TOUCH_RIGHT) | (1 << TOUCH_TOP) | (1 << TOUCH_BOTTOM));
	DDRA |= (1 << TOUCH_GND);
	PORTA &= ~(1 << TOUCH_GND);
	PORTA |= (1 << TOUCH_SAMPLE);
	PORTD = (1 << PUSHBUTTON1) | (1 << PUSHBUTTON2);
	EEPROM_readTouchData(tmp);
	lowerX = tmp[0];
	lowerY = tmp[1];
	upperX = tmp[2];
	upperY = tmp[3];
	touched = 0;
	debounceButton1 = 0;
	debounceButton2 = 0;
	debounceTouch = 0;
	setupButtons();
	if (lowerX > 0 && lowerY > 0 && upperX > 0 && upperY > 0) return 0;
	return 2;
}

void Touch_clearAll(void) {
	int16_t tmp[4];
	tmp[0] = 0xffff;
	tmp[1] = 0xffff;
	tmp[2] = 0xffff;
	tmp[3] = 0xffff;
	EEPROM_writeTouchData(tmp);
}

uint8_t Touch_readButton(void) {
	uint8_t bx, by;
	debounceButton1 <<= 1;
	debounceButton1 |= (PIND & (1 << PUSHBUTTON1)) == 0 ? 1 : 0;
	if (debounceButton1 == 0x0f) {
		return 12;
	}
	debounceButton2 <<= 1;
	debounceButton2 |= (PIND & (1 << PUSHBUTTON2)) == 0 ? 1 : 0;
	if (debounceButton2 == 0x0f) {
		return 13;
	}
	if (touched) {
		touched = 0;
		if (lastX < lowerX || lastX > upperX || lastY < lowerY || lastY > upperY) return 0xff;
		if (lastX < buttonX1) {
			bx = 0;
		} else if (lastX < buttonX2) {
			bx = 1;
		} else if (lastX < buttonX3) {
			bx = 2;
		} else {
			bx = 3;
		}
		if (lastY < buttonY1) {
			by = 0;
		} else if (lastY < buttonY2) {
			by = 4;
		} else {
			by = 8;
		}
		return by + bx;
	}
	return 0xff;
}

void Touch_setUpperLeft(void) {
	lowerX = lastX;
	lowerY = lastY;
	touched = 0;
}

void Touch_setLowerRight(void) {
	upperX = lastX;
	upperY = lastY;
	touched = 0;
}

uint8_t Touch_save(void) {
	int16_t tmp[4];
	touched = 0;
	if (abs(upperX - lowerX) > 100 && abs(upperY - lowerY) > 100) {
		tmp[0] = lowerX;
		tmp[1] = lowerY;
		tmp[2] = upperX;
		tmp[3] = upperY;
		EEPROM_writeTouchData(tmp);
		setupButtons();
		return 0;
	}
	return 2;
}

uint8_t Touch_getTouch(void) {
	uint8_t repeat;
	debounceTouch <<= 1;
	debounceTouch |= (PINA & (1 << TOUCH_TOP)) == 0 ? 1 : 0;
	if (touched == 0 && debounceTouch == 0x0f) {
		DDRA |= (1 << TOUCH_LEFT) | (1 << TOUCH_RIGHT);
		DDRA &= ~((1 << TOUCH_TOP) | (1 << TOUCH_BOTTOM));
		PORTA &= ~(1 << TOUCH_LEFT);
		PORTA |= (1 << TOUCH_RIGHT);
		ADMUX = (0 << REFS1) | (1 << REFS0) | (0 << ADLAR) | 0x03;
		lastX = 0;
		for (repeat = 0; repeat < AVERAGE; repeat++) {
			ADCSRA = (1 << ADEN) | (1 << ADSC) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);
			while ((ADCSRA & (1 << ADSC)) != 0);
			lastX += ADC;
		}
		lastX /= AVERAGE;
		DDRA |= (1 << TOUCH_TOP) | (1 << TOUCH_BOTTOM);
		DDRA &= ~((1 << TOUCH_LEFT) | (1 << TOUCH_RIGHT));
		PORTA &= ~(1 << TOUCH_TOP);
		PORTA |= (1 << TOUCH_BOTTOM);
		ADMUX = (0 << REFS1) | (1 << REFS0) | (0 << ADLAR) | 0x02;
		lastY = 0;
		for (repeat = 0; repeat < AVERAGE; repeat++) {
			ADCSRA = (1 << ADEN) | (1 << ADSC) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);
			while ((ADCSRA & (1 << ADSC)) != 0);
			lastY += ADC;
		}
		lastY /= AVERAGE;
		ADCSRA = (0 << ADEN) | (0 << ADSC) | (0 << ADPS2) | (0 << ADPS1) | (0 << ADPS0);
		DDRA &= ~((1 << TOUCH_LEFT) | (1 << TOUCH_RIGHT) | (1 << TOUCH_TOP) | (1 << TOUCH_BOTTOM));
		DDRA |= (1 << TOUCH_GND);
		PORTA &= ~(1 << TOUCH_GND);
		PORTA |= (1 << TOUCH_SAMPLE);
		touched = 1;
		return 1;
	}
	return 0;
}

void Touch_setup(void) {
	uint8_t set = 2;

	LED_setColor(255, 100, 100);
	LCD_showUpperLeft();
	LCD_loop(1);
	while (set > 0) {
		if (Touch_getTouch()) {
			if (set == 2) {
				Touch_setUpperLeft();
				set--;
				LCD_showLowerRight();
				LCD_loop(1);
				LED_setColor(100, 255, 100);
			} else if (set == 1) {
				Touch_setLowerRight();
				set--;
				set = Touch_save();
				if (set == 2) {
					LED_setColor(255, 100, 100);
					LCD_showUpperLeft();
					LCD_loop(1);
				}
			}
		}
	}
	LED_setColor(255, 255, 255);
	LCD_hideCorners();
	LCD_loop(1);
}
